function hyperClassif= newHyperClassifier(samples, classIds, config)
    
    % get nClasses and list of ids
    hyperClassif.classIds= [];
    hyperClassif.nClasses= 0;
    minId= min(classIds);
    maxId= max(classIds);
    for id= minId : maxId
        if any(classIds == id) && all(hyperClassif.classIds ~= id)
            hyperClassif.nClasses= hyperClassif.nClasses + 1;
            hyperClassif.classIds(hyperClassif.nClasses)= id;
        end
    end
    
    % get sample dimensions
    nSamples= length(samples);
    
    % init hierarchy of classifiers
    nLevels_= zeros(1, nSamples);
    for sam= 1 : nSamples
        nLevels_(sam)= size(samples{sam}, 1);
    end
    nLevels= max([nLevels_ config.nHierarchyLevels]);
    filterClassifier= config.filterClassifier;
    classifiers= newClassifier(1, 1, config.classifierConfigs(1));
    nPrototypes= filterClassifier.nClasses;
    classifierConfigs= config.classifierConfigs;
    featureParams= config.featureParams;
    gCCs= zeros(hyperClassif.nClasses + 1, nPrototypes, nLevels); % globalCenterCredits
    cDRs= zeros(hyperClassif.nClasses, nPrototypes, nLevels, 2);  % centerDistanceRatios
    classDescriptions= cell(1, nLevels);
    classOccurances= zeros(1, hyperClassif.nClasses + 1);
    
    for sam= 1 : nSamples
        if mod(sam, 10) == 0, fprintf('.'); if mod(sam, 100) == 0, fprintf('\n'); end, end
        % generate base-segment info (transInfo, start, end)
        % ASSUMPTION: closed curves, with transInfo end-begin at 1st point
        classIndex= find(hyperClassif.classIds == classIds(sam));
        sa= samples{sam};
        segments= sa.segments;
        [nSegLevels, nSegs]= size(sa.segments);
%         if nSegLevels ~= nLevels, warning(['newHyperClassifier: sample ' num2str(sam) ': # segmentation level does not equal # classifiers!']); end
        level= 1;
        
        feats= zeros(sum(featureParams(level).nFeatures), nSegs);
        for seg= 1 : nSegs
            feats(:, seg)= getFeatures(sa, level, seg, featureParams(level).nFeatures, 25);
        end
        
        % classify segments with filterClassifier
        if strcmp(filterClassifier.config.type, 'classifier')
            segTypes= getClassifierResponse(filterClassifier, feats);
        elseif strcmp(filterClassifier.config.type, 'ensemble')
            segTypes= getEnsembleClassifierResponse(filterClassifier, feats);
        end
        
        
        for seg= 1 : nSegs
            i2= mod(seg-2^(level-1), nSegs);    % index for seg, where this is 2nd subType
            if i2 == 0, i2= nSegs; end
            i3= mod(seg-2^level, nSegs);        % index for seg, where this is 3rd subType
            if i3 == 0, i3= nSegs; end
            segments(level, seg).type= segTypes(seg);
            segments(level+1, seg).subTypes(1)= segTypes(seg);
            segments(level+1, i2).subTypes(2)= segTypes(seg);
            segments(level+1, i3).subTypes(3)= segTypes(seg);
        end
        
        % update globalCenterCredits and centerDistanceRatios
        for proto= 1 : nPrototypes
            inds= find(proto == segTypes);
            nHits= length(inds); if nHits == 0, continue; end
            cDRs(classIndex, proto, level, :)= cDRs(classIndex, proto, level, :) .* ...
                gCCs(1 + classIndex, proto, level);
            for i= 1 : nHits
                ind= inds(i);
                cDRs(classIndex, proto, level, 1)= cDRs(classIndex, proto, level, 1) + ...
                    segments(level, ind).dists(1);
                cDRs(classIndex, proto, level, 2)= cDRs(classIndex, proto, level, 2) + ...
                    segments(level, ind).dists(2);
            end
            gCCs(1 + classIndex, proto, level)= gCCs(1 + classIndex, proto, level) + nHits;
            gCCs(1, proto, level)= gCCs(1, proto, level) + nHits;
            cDRs(classIndex, proto, level, :)= cDRs(classIndex, proto, level, :) ./ ...
                gCCs(1 + classIndex, proto, level);
        end
        
        
        for level= 2 : min([nSegLevels nLevels])
            
            feats= zeros(sum(featureParams(level).nFeatures), nSegs);
            for seg= 1 : nSegs
                feats(:, seg)= getFeatures(sa, level, seg, featureParams(level).nFeatures, 25);
            end
            
            % train next classifier
            % ASSUMPTION: classifier can treat classIds == 0 (rubbish,
            % background class)
            [segTypes, classDescriptions{level}]= generateTrainIds(classDescriptions{level}, segments, level);
            
            if length(classifiers) >= level
%                 classifiers(level)= updateClassifier(classifiers(level), feats, segTypes);
                [classifiers(level), segTypes]= updateClassifier(classifiers(level), feats, segTypes);
            else
%                 classifiers(level)= newClassifier(feats, segTypes, classifierConfigs(level));
                [classifiers(level), segTypes]= newClassifier(feats, segTypes, [], classifierConfigs(level));
            end
            
            % update globalCenterCredits and centerDistanceRatios
            % [nCla, nCen, nLev]= size(gCCs);

            nCen = size(gCCs, 2);
            
            maxId= max(segTypes);
            if nCen < maxId
                gCCs(1 + classIndex, maxId, level)= 0; 
                cDRs(classIndex, maxId, level, 1:2)= 0;
            end
            for proto= 1 : maxId
                inds= find(proto == segTypes);
                nHits= length(inds); if nHits == 0, continue; end
                cDRs(classIndex, proto, level, :)= cDRs(classIndex, proto, level, :) .* ...
                    gCCs(1 + classIndex, proto, level);
                for i= 1 : nHits
                    ind= inds(i);
                    cDRs(classIndex, proto, level, 1)= cDRs(classIndex, proto, level, 1) + ...
                        segments(level, ind).dists(1);
                    cDRs(classIndex, proto, level, 2)= cDRs(classIndex, proto, level, 2) + ...
                        segments(level, ind).dists(2);
                end
                gCCs(1 + classIndex, proto, level)= gCCs(1 + classIndex, proto, level) + nHits;
                gCCs(1, proto, level)= gCCs(1, proto, level) + nHits;
                cDRs(classIndex, proto, level, :)= cDRs(classIndex, proto, level, :) ./ ...
                    gCCs(1 + classIndex, proto, level);
            end
            
            if level == nLevels, continue; end
            for seg= 1 : nSegs
                i2= mod(seg-2^(level-1), nSegs);    % index for seg, where this is 2nd subType
                if i2 == 0, i2= nSegs; end
                i3= mod(seg-2^level, nSegs);        % index for seg, where this is 3rd subType
                if i3 == 0, i3= nSegs; end
                segments(level, seg).type= segTypes(seg);
                segments(level+1, seg).subTypes(1)= segTypes(seg);
                segments(level+1, i2).subTypes(2)= segTypes(seg);
                segments(level+1, i3).subTypes(3)= segTypes(seg);
            end
        end % level
        
        classOccurances(1)= classOccurances(1) + 1;
        classOccurances(classIndex + 1)= classOccurances(classIndex + 1) + 1;
    end % sam
    
    
    hyperClassif.config= config;
    hyperClassif.nLevels= nLevels;
    hyperClassif.filterClassifier= filterClassifier;
    hyperClassif.classifiers= classifiers;
    hyperClassif.nPrototypes= nPrototypes;
    hyperClassif.classifierConfigs= classifierConfigs;
    hyperClassif.featureParams= featureParams;
    hyperClassif.globalCenterCredits= gCCs;
    hyperClassif.centerDistanceRatios= cDRs;
    hyperClassif.classDescriptions= classDescriptions;
    hyperClassif.classOccurances= classOccurances;
    
    