function classifier= newClassifier_(samples, classIds, weights, config)

    [nDims, nSamples]= size(samples);
    
    % initialize parameters
    if nargin < 3 || isempty(weights), weights= ones(1, nSamples); end
    if nargin < 4, config= createClassifierConfig_(); end
    classifier.config= config;
    
    if config.displayInfo == 1, fprintf('samples to be trained: %d\n', nSamples); end
    
    %nSinceLastReorg= 0;
    nReorganizeSamples= classifier.config.nReorganizeSamples;
    reorganize= classifier.config.reorganize;
    growthThresh= classifier.config.growthThresh;
    growthFactor= classifier.config.growthFactor;
    shrinkThresh= classifier.config.shrinkThresh;
    vanishThresh= classifier.config.vanishThresh;
    newClusterThresh= classifier.config.newClusterThresh;
    scale= classifier.config.scale;
    keepHistory= config.keepHistory;
    
    % get nClasses and list of ids
    classifier.classIds= [];
    classifier.nClasses= 0;
    minId= min(classIds);
    maxId= max(classIds);
    for id= minId : maxId
        if any(classIds == id) && all(classifier.classIds ~= id)
            classifier.nClasses= classifier.nClasses + 1;
            classifier.classIds(classifier.nClasses)= id;
        end
    end
    
    
    % permute samples + ids
    if classifier.config.permuteSamples
        indices= randperm(nSamples);
        samples= samples(:, indices);
        classIds= classIds(indices);
        weights= weights(indices);
    end
    
    % estimate scale, i.e. distance between samples
    if scale == -1
        indices= randperm(nSamples);
        samples_= samples(:, indices);
        a= min([200 nSamples]);
        dd= pdist(samples_(:, 1:a)');
        if length(dd) > 1
            edges= min(dd) : (max(dd)-min(dd))/sqrt(length(dd)) : max(dd);
            n= histc(dd, edges);
    % 	    figure;bar(edges, n);
            mass= sum(n);
            for i= 1 : length(n)
                if sum(n(1:i))/mass > classifier.config.scalePercentage, break; end
            end
            if i == 1, i= 2; end
            scale= edges(i);
        end
        if isempty(scale) || scale == 0, scale= 1; end
        classifier.config.scale= scale;
    end
    
    % initialize models
    centers= samples(:, 1);
    radii= scale;
    centerClassIds= classIds(1);
    nCenters= 1;
    correctResponses= weights(1);
    totalResponses= weights(1);
    credits= 1;
    centerIds= 1 : nCenters;
    nSinceLastReorg= 1;
    
    if keepHistory == 1
        nCenterHist= zeros(1, nSamples);
        nCenterHist(1)= nCenters;
        radiiHist= zeros(nSamples, 100);
        radiiHist(1, 1:length(radii))= radii;
        creditHist= zeros(nSamples, 100);
        creditHist(1, 1:length(credits))= credits;
        nRespHist= zeros(nSamples, 100);
        nRespHist(1, 1:length(totalResponses))= totalResponses;
        ccIdHist= zeros(nSamples, 100);
        ccIdHist(1, 1:length(centerClassIds))= centerClassIds;
        centerHist= zeros(nDims, 100, nSamples);
        centerHist(1 : size(centers), 1)= centers;
        centerDiffHist= zeros(nDims, 100, nSamples);
    end
   
    for sam= 2 : nSamples
%         if mod(sam, 100) == 0
%             fprintf('.'); 
%             if mod(sam, 1000) == 0, fprintf('\n'); end 
%         end
        % calculate responses for all clusters
        sampleMatrix= samples(:, ones(1, nCenters).*sam);
        diffVecs= sampleMatrix - centers;
        distSquared= sum(diffVecs.^2);
        if nDims == 1, distSquared= diffVecs.^2; end
        responses= 1 - distSquared ./ (radii.^2);
%         responses= 1 - sum((centers - sampleMatrix).^2) ./ (radii).^2;

        % compute indices
        cis= centerClassIds == classIds(sam);  % indices of correct clusters
        fis= cis == 0;  % indices of foreign clusters
        ris= responses > 0; % indices of responding clusters
        nris= ris == 0; % indices of not responding clusters
        cris= logical(cis .* ris); % correctly responding cluster indices
        cnris= logical(cis .* nris); % indices of correct clusters that have not responded
        fris= logical(fis .* ris); % falsely responding cluster indices
        
        % update falsely responding clusters
        totalResponses(fris)= totalResponses(fris) + weights(sam);
        credits(fris)= correctResponses(fris) ./ totalResponses(fris);
        
        % update correctly responding clusters
        correctResponses(cris)= correctResponses(cris) + weights(sam);
        totalResponses(cris)= totalResponses(cris) + weights(sam);
        credits(cris)= correctResponses(cris) ./ totalResponses(cris);
        
        % update correct but not responding clusters
        cnris(credits < growthThresh)= 0; % that may be grown
        responses_= 1 - distSquared(cnris) ./ (radii(cnris) .* ((growthFactor-1).*weights(sam)+1)).^2;
        ctbg= centerIds(cnris);
        ctbg(responses_ < 0)= []; % indices of clusters to be grown
        if sum(ctbg) > 0
            dists= sqrt(distSquared(ctbg));
            offsets= (dists - radii(ctbg)) ./ 2;
            % work around for: offsetVecs= offsets .* diffVecs(:, ctbg) ./ dists;
            temp= offsets ./ dists;
            tempMatrix= temp(ones(1, nDims), :);
            offsetVecs= diffVecs(:, ctbg) .* tempMatrix;
            centers(:, ctbg)= centers(:, ctbg) + offsetVecs;
            if keepHistory == 1
                paddedOffsetVecs= zeros(size(centers));
                paddedOffsetVecs(:, ctbg)= offsetVecs;
                centerDiffHist(1:size(centers, 1), 1:size(centers, 2), sam)= paddedOffsetVecs;
            end
            radii(ctbg)= radii(ctbg) + offsets;
            correctResponses(ctbg)= correctResponses(ctbg) + weights(sam);
            totalResponses(ctbg)= totalResponses(ctbg) + weights(sam);
            credits(ctbg)= correctResponses(ctbg) ./ totalResponses(ctbg);
        end
        
        % calculate responses for all clusters
        % - not necessary, as responses stay the same (ctbg are 0)
        
        % sum responses of member clusters
        support= sum(responses(cris) .* credits(cris));
        
        % if necessary initialize new cluster
        if support <= newClusterThresh
            nCenters= nCenters + 1;
            centers(:, nCenters)= samples(:, sam);
            centerClassIds(nCenters)= classIds(sam);
            correctResponses(nCenters)= weights(sam);
            totalResponses(nCenters)= weights(sam);
            credits(nCenters)= 1;
            centerIds= 1 : nCenters;
            % scale adaptation
            if strcmp(config.scaleAdaptation, 'none')
                radii(nCenters)= scale;
            elseif strcmp(config.scaleAdaptation, 'allMean')
                scale= mean(radii);
                radii(nCenters)= scale;
            elseif strcmp(config.scaleAdaptation, 'neighbourMean')
                allMean= mean(radii);
                neighbourIndices= sqrt(distSquared) <= (2 * allMean);
                if sum(neighbourIndices) > 1
                    radii(nCenters)= mean(radii(neighbourIndices));
                else
                    radii(nCenters)= allMean;
                end
            end
        end
        
        % shrink/delete cluster
        if sum(fris) > 0
            sis= credits < shrinkThresh; % indices of clusters below shrinkThresh
            sis= logical(sis(1:length(fris)) .* fris);
            radii(sis)= radii(sis) ./ growthFactor;
            
            dis= credits < vanishThresh; % indices of clusters below vanishThresh
            if sum(dis) > 0
                dis= logical(dis(1:length(fris)) .* fris);
                centers(:, dis)= [];
                radii(dis)= [];
                centerClassIds(dis)= [];
                correctResponses(dis)= [];
                totalResponses(dis)= [];
                credits(dis)= [];
                nCenters= nCenters - sum(dis);
            end
        end
        nSinceLastReorg= nSinceLastReorg + 1;
        if reorganize == 1 && nSinceLastReorg >= nReorganizeSamples
            classifier.nCenters= nCenters;
            classifier.centers= centers;
            classifier.radii= radii;
            classifier.centerClassIds= centerClassIds;
            classifier.correctResponses= correctResponses;
            classifier.totalResponses= totalResponses;
            classifier.credits= credits;
            classifier.nSamples= nSamples;
            
            classifier= reorganizeClassifier(classifier);
            
            centers= classifier.centers;
            radii= classifier.radii;
            centerClassIds= classifier.centerClassIds;
            nCenters= classifier.nCenters;
            correctResponses= classifier.correctResponses;
            totalResponses= classifier.totalResponses;
            credits= classifier.credits;
            centerIds= 1 : nCenters;
            nSinceLastReorg= classifier.nSinceLastReorg;
        end
        if sum(isnan(radii)) > 0, error('newClassifier: radii are NaN!!'); end
        if keepHistory == 1
            nCenterHist(sam)= nCenters;
            radiiHist(sam, 1:length(radii))= radii;
            creditHist(sam, 1:length(credits))= credits;
            nRespHist(sam, 1:length(totalResponses))= totalResponses;
            ccIdHist(sam, 1:length(centerClassIds))= centerClassIds;
            centerHist(1:size(centers, 1), 1:size(centers, 2), sam)= centers;
        end
    end
    classifier.nDims= nDims;
    classifier.nCenters= nCenters;
    classifier.centers= centers;
    classifier.radii= radii;
    classifier.centerClassIds= centerClassIds;
    classifier.correctResponses= correctResponses;
    classifier.totalResponses= totalResponses;
    classifier.credits= credits;
    classifier.nSamples= nSamples;
    classifier.nSinceLastReorg= nSinceLastReorg;
    classifier.history= [];
    
    if keepHistory == 1
        classifier.history.nCenters= nCenterHist;
        classifier.history.centers= centerHist;
        classifier.history.centerDiffs= centerDiffHist;
        classifier.history.radii= radiiHist;
        classifier.history.credits= creditHist;
        classifier.history.totalResponses= nRespHist;
        classifier.history.centerClassIds= ccIdHist;
    end